//******************************************************************************
//  Demo Application05 for MSP430/LMP91200 Interface Code Library
//  2-step temperature measurement using ADC12 on MSP430F5528
//  LMP91200 VOUT is connected to Channel A0 of MSP430F5528.
//
//                MSP430F5528
//             -----------------
//         /|\|              XIN|-
//          | |                 |  
//          --|RST          XOUT|-
//            |                 |
//            |    P3.3/UCA0SIMO|--> SDI 
//            |    P3.4/UCA0SOMI|<-- SDO 
//            |     P2.7/UCA0CLK|--> CLK 
//            |             P2.5|--> CSB 
//            |                 | 
//            |     P6.0/CB0/A0 |<-- VOUT 
//            |                 | 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   March 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************
/*  Copyright 2011-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is 
  integrated into your product. Other than for the foregoing purpose, you may 
  not use, reproduce, copy, prepare derivative works of, modify, distribute, 
  perform, display or sell this Software and/or its documentation for any 
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91200.h"
#include "TI_LMP91200_register_settings.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_spi.h"

void ADC12_Init(void);                                                         // To init MSP430F5528 ADC12 & Start Conversion
#define TI_LMP91200_VOUT_ADC12_PxSEL      P6SEL                                // ADC12 A0 channel on P6.0 (MSP430F5528)
#define TI_LMP91200_VOUT_ADC12_PIN        BIT0

#define NUM_OF_RESULTS      8                                                  // Number of temp sensor samples to take
#define SCALE_FACTOR        3                                                  // For averaging converted samples
#define PT_RES_NOMINAL      100                                                // 100ohms at 0degC
#define ADC12_RATIO         0.6105                                             // 2500/4095 (2.5V reference & 12bit converter)
#define ADC14_RATIO         0.15259                                            // 2500/16384(2500mV ref & 14bit converter w/ 2's comp o/p)
#define PT_TC               0.3850                                             // PT_TC is ohms/degC
#define PGA_GAIN            10                                                 // Gain of 10
#define I_VALUE             1                                                  // 1mA 
#define R_REF               100                                                // 100ohm reference resistor
//******************************************************************************
void main(void)
{
  uint16_t reg_write_data, reg_read_data;
    
  WDTCTL = WDTPW+WDTHOLD;                                                      // Stop WDT
  
  TI_LMP91200_LED_PxOUT |= TI_LMP91200_LED_PIN;                                // Set LED ON
  TI_LMP91200_LED_PxDIR |= TI_LMP91200_LED_PIN;                                // Set pin direction is output
  
  TI_LMP91200_SPISetup();                                                      // Initilaize MSP430 SPI Block
  
  reg_write_data = TI_LMP91200_CONFIG_REG_VALUE;                               // value to write 
  reg_read_data = TI_LMP91200_SPIWriteReg(reg_write_data);                     // Write again to read config register  
  reg_read_data = TI_LMP91200_SPIWriteReg(reg_write_data);                     // Write again to read config register
  // test if write/read values match
  if (reg_write_data != reg_read_data)
  {
   while (1)                                                                   // error: blink LED continuously
   {
     __delay_cycles(250000);
     TI_LMP91200_LED_PxOUT ^= TI_LMP91200_LED_PIN;        
   }      
  }
  
  TI_LMP91200_SPIWriteReg(TI_LMP91200_CONFIG_REG_RCAL_VALUE);                  // Measure RCAL
  ADC12_Init();                                                                // Initialize MSP430F5528 ADC12 & Start Conversion
     
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0, Enable interrupts
  __no_operation();                                                            // For debugger  
                                                                                 
}
//------------------------------------------------------------------------------
// Description:
//   Initialization of the ADC12 Module (MSP430F5528) & Start Conversion
// Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
void ADC12_Init(void)
{
  TI_LMP91200_VOUT_ADC12_PxSEL |= TI_LMP91200_VOUT_ADC12_PIN;                  // Enable A/D channel A0
  REFCTL0 &= ~REFMSTR;                                                         // Reset REFMSTR to hand over control to 
                                                                               // ADC12_A ref control registers  
  ADC12CTL0 = ADC12ON+ADC12SHT0_8+ADC12MSC;                                    // Turn on ADC12, set sampling time
                                                                               // set multiple sample conversion
  ADC12CTL0 |= ADC12REFON+ADC12REF2_5V;                                        // Turn on Ref Gen & set to 2.5V
  ADC12MCTL0 = ADC12SREF_1;                                                    // Vr+ = Vref+ and Vr- = AVSS
  ADC12CTL1 = ADC12SHP+ADC12CONSEQ_2;                                          // Use sampling timer, set mode
  ADC12IE = ADC12IE0;                                                          // Enable ADC12IFG.0
  __delay_cycles(500);                                                         // delay to allow Ref to settle
  ADC12CTL0 |= ADC12ENC;                                                       // Enable conversions
  ADC12CTL0 |= ADC12SC;                                                        // Start conversion
  
}
//------------------------------------------------------------------------------
// Description:
//   ADC12 Interrupt Service Routine (MSP430F5528)
// Note: This ISR is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
#pragma vector=ADC12_VECTOR
__interrupt void ADC12ISR (void)
{
  static int8_t index = 0;
  static uint8_t RCAL_FLAG = 1;                                                // RCAL or RTD measurement indicator flag
  volatile static uint16_t results[NUM_OF_RESULTS];                            // To store ADC12 output
  static uint32_t sum_adc_data = 0;                                            // to accumulate and avg adc results
  static float vout, itrue, pt_res_calc;                                       // voltage output, true current, calculated resistance (PT100)
  volatile static float measured_temp;                                         // 2step measured temperature

  switch(__even_in_range(ADC12IV,34))
  {
  case  0: break;                                                              // Vector  0:  No interrupt
  case  2: break;                                                              // Vector  2:  ADC overflow
  case  4: break;                                                              // Vector  4:  ADC timing overflow
  case  6:                                                                     // Vector  6:  ADC12IFG0
    results[index] = ADC12MEM0;                                                // Move & save results
    sum_adc_data += ADC12MEM0;                                                 // accumulate result values
    index++;                                                                   // Increment results index, modulo;    
    if (index == NUM_OF_RESULTS)
    {
      sum_adc_data >>= SCALE_FACTOR;                                           // Divide by NUM_OF_RESULTS
      vout = ADC12_RATIO * sum_adc_data;
      if (RCAL_FLAG)
      {
        ADC12CTL1 = ADC12CONSEQ_0;                                             // Reset mode    
        ADC12CTL0 &= ~ADC12SC;                                                 // Stop conversion
        ADC12CTL0 &= ~ADC12ENC;                                                // Disable conversions
        ADC12IE &= ~ADC12IE0;                                                  // Disable ADC12IE0 interrupt        
        ADC12IFG &= ~ADC12IFG0;                                                // clear flag
        itrue = vout/(R_REF * PGA_GAIN);                                       // LMP91200 DS, pg 19, eqn 6
        TI_LMP91200_SPIWriteReg(TI_LMP91200_CONFIG_REG_RTD_VALUE);             // Measure RTD
        RCAL_FLAG = 0;                                                         // clear flag
        ADC12CTL1 = ADC12SHP+ADC12CONSEQ_2;                                    // Use sampling timer, set mode        
        ADC12CTL0 |= ADC12ENC;                                                 // Enable conversions
        ADC12CTL0 |= ADC12SC;                                                  // Start conversion  
        ADC12IE = ADC12IE0;                                                    // Enable ADC12IE0 interrupt        
      }
      else
      {
        ADC12CTL1 = ADC12CONSEQ_0;                                             // Stop conversion
        ADC12CTL0 &= ~ADC12SC;                                                 // Stop conversion        
        ADC12CTL0 &= ~ADC12ENC;                                                // Disable conversions
        ADC12IE &= ~ADC12IE0;                                                  // Disable ADC12E0 interrupt         
        ADC12IFG &= ~ADC12IFG0;                                                // clear flag
        pt_res_calc = vout/(itrue * PGA_GAIN);                                 // LMP91200 DS, pg 19 eqn 5
        measured_temp = (pt_res_calc - PT_RES_NOMINAL) / PT_TC;                // LMP91200 DS pg 19 equation 4        
        TI_LMP91200_SPIWriteReg(TI_LMP91200_CONFIG_REG_RCAL_VALUE);            // Measure RCAL     
        RCAL_FLAG = 1;                                                         // set flag
        ADC12CTL1 = ADC12SHP+ADC12CONSEQ_2;                                    // Use sampling timer, set mode
        ADC12CTL0 |= ADC12ENC;                                                 // Enable conversions
        ADC12CTL0 |= ADC12SC;                                                  // Start conversion         
        ADC12IE = ADC12IE0;                                                    // Enable ADC12IE0 interrupt
      }
      sum_adc_data = 0;                                                        // Set Breakpoint here & see measured temperature      
      index = 0;
    }  
    break; 
  case  8: break;                                                              // Vector  8:  ADC12IFG1
  case 10: break;                                                              // Vector 10:  ADC12IFG2
  case 12: break;                                                              // Vector 12:  ADC12IFG3
  case 14: break;                                                              // Vector 14:  ADC12IFG4
  case 16: break;                                                              // Vector 16:  ADC12IFG5
  case 18: break;                                                              // Vector 18:  ADC12IFG6
  case 20: break;                                                              // Vector 20:  ADC12IFG7
  case 22: break;                                                              // Vector 22:  ADC12IFG8
  case 24: break;                                                              // Vector 24:  ADC12IFG9
  case 26: break;                                                              // Vector 26:  ADC12IFG10
  case 28: break;                                                              // Vector 28:  ADC12IFG11
  case 30: break;                                                              // Vector 30:  ADC12IFG12
  case 32: break;                                                              // Vector 32:  ADC12IFG13
  case 34: break;                                                              // Vector 34:  ADC12IFG14
  default: break; 
  }
}
//------------------------------------------------------------------------------
