//******************************************************************************
//  Description:  This file contains functions that allow the MSP430 device to
//  access the SPI interface of the LMP91200.  There are multiple
//  instances of each function; the one to be compiled is selected by the
//  system variable TI_LMP91200_SER_INTF, defined in "TI_LMP91200_hardware_board.h".
//
//  MSP430/LMP91200 Interface Code Library v1.0
// 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   March 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************
/*  Copyright 2011-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is 
  integrated into your product. Other than for the foregoing purpose, you may 
  not use, reproduce, copy, prepare derivative works of, modify, distribute, 
  perform, display or sell this Software and/or its documentation for any 
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91200.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_spi.h"


//******************************************************************************
// Support for 261x USCI_A0
//******************************************************************************
#if TI_LMP91200_SER_INTF == TI_LMP91200_SER_INTF_USCIA0
//------------------------------------------------------------------------------
//  void TI_LMP91200_SPISetup(void)
//
//  DESCRIPTION:
//  Configures the assigned interface to function as a SPI port and
//  initializes it.
//------------------------------------------------------------------------------
void TI_LMP91200_SPISetup(void)
{
  TI_LMP91200_CSn_PxOUT |= TI_LMP91200_CSn_PIN;
  TI_LMP91200_CSn_PxDIR |= TI_LMP91200_CSn_PIN;                                // /CS disable

  UCA0CTL1 |= UCSWRST;                                                         // **Disable USCI state machine**
  UCA0CTL0 |= UCMST+UCCKPL+UCMSB+UCSYNC;                                       // 3-pin, 8-bit SPI master 
  UCA0CTL1 |= UCSSEL_2;                                                        // SMCLK
  UCA0BR0 = 0x04;                                                              // UCLK/4
  UCA0BR1 = 0;
  UCA0MCTL = 0;
  TI_LMP91200_SPI_UCA0_PxSEL1 |= TI_LMP91200_SPI_UCA0_SIMO
                                   | TI_LMP91200_SPI_UCA0_SOMI;              
  TI_LMP91200_SPI_UCA0_PxSEL2 |= TI_LMP91200_SPI_UCA0_UCLK;
                                                                               // SPI option select
  TI_LMP91200_SPI_UCA0_PxDIR1 |= TI_LMP91200_SPI_UCA0_SIMO;
  TI_LMP91200_SPI_UCA0_PxDIR2 |= TI_LMP91200_SPI_UCA0_UCLK;
                                                                               // SPI TXD out direction
  
  UCA0CTL1 &= ~UCSWRST;                                                        // **Initialize USCI state machine**
}

//------------------------------------------------------------------------------
//  uint16_t TI_LMP91200_SPIWriteReg(uint16_t value)
//
//  DESCRIPTION:
//  Writes "value" to configuration register. Returns the previous value
//  in the configuration register.
//------------------------------------------------------------------------------
uint16_t TI_LMP91200_SPIWriteReg(uint16_t value)
{
  uint16_t x=0;
  uint8_t inst;
  
  TI_LMP91200_CSn_PxOUT &= ~TI_LMP91200_CSn_PIN;                               // /CS enable 
  
  inst = value>>8;                                                             // Get msb data to send
  
  while (!(IFG2&UCA0TXIFG));                                                   // Wait for TXBUF ready
  UCA0TXBUF = inst;                                                            // Send msb   
  
  while (UCA0STAT & UCBUSY);                                                   // Wait for TX complete
  x = UCA0RXBUF;                                                               // Read previous msb
  
  inst = value & 0x00FF;                                                       // Get lsb data to send
  while (!(IFG2&UCA0TXIFG));                                                   // Wait for TXBUF ready
  UCA0TXBUF = inst;                                                            // Send lsb data value   
  
  while (UCA0STAT & UCBUSY);                                                   // Wait for TX complete
  x = (x << 8) | UCA0RXBUF;                                                    // Read previous lsb & append 
  
  TI_LMP91200_CSn_PxOUT |= TI_LMP91200_CSn_PIN;                                // /CS disable
  
  return x;                                                                    // return previous register value  
  
}
//------------------------------------------------------------------------------
#endif

